#include <fmt/format.h>
#include <iostream>
#include <string>

/**

    {} or {:p} : person as [name, age]
    {:n} : name only
    {:a} : age only
*/

struct Person
{
    std::string name;
    int age;
};

template<>
struct fmt::formatter<Person>
{
    auto constexpr parse(fmt::format_parse_context& ctx)
    {
        auto it{ctx.begin()};
        if(it == ctx.end() || *it == '}')
        {
            mMode = Mode::Person;
            return it;
        }
        switch(*it)
        {
        case 'p':
            mMode = Mode::Person;
            break;
        case 'n':
            mMode = Mode::Name;
            break;
        case 'a':
            mMode = Mode::Age;
            break;
        default:
            throw fmt::format_error{"Invalid format specifier."};
        }
        ++it;
        if(it != ctx.end() && *it != '}') /// there is still stuff we do not expect
        {
            throw fmt::format_error{"Invalid format specifier."};
        }
        return it;
    }

    auto format(const Person& person, fmt::format_context& ctx)
    {
        auto out = ctx.out();
        switch(mMode) // not using c++20 using for the num (using enum Mode)
        {
        case Mode::Person:
            return fmt::format_to(out, "[{}, {}]\n", person.name, person.age);
        case Mode::Name:
            return fmt::format_to(out, "{}\n", person.name);
        case Mode::Age:
            return fmt::format_to(out, "{}\n", person.age);
        }
        return fmt::format_to(out, "");
    }
private:
    enum class Mode
    {
        Person,
        Name,
        Age
    };
    Mode mMode;
};

int main()
{
    Person p1{"Prince", 51};

    std::cout << fmt::format("Person: {}", p1) << std::endl;
    std::cout << fmt::format("Person: {:p}", p1) << std::endl;
    std::cout << fmt::format("Person: {:n}", p1) << std::endl;
    std::cout << fmt::format("Person: {:a}", p1) << std::endl;

    try
    {
        std::cout << fmt::format("Person: {:q}", p1) << std::endl;
    }
    catch(const fmt::format_error& exc)
    {
        std::cout << exc.what() << std::endl;
    }

    try
    {
        std::cout << fmt::format("Person: {:nope}", p1) << std::endl;
    }
    catch(const fmt::format_error& exc)
    {
        std::cout << exc.what() << std::endl;
    }

    return 0;
}
